@@ -1,5 +1,7 @@
+from __future__ import (absolute_import, print_function, division)
 import re, copy
 
+
 def safe_subn(pattern, repl, target, *args, **kwargs):
     """
         There are Unicode conversion problems with re.subn. We try to smooth
@@ -22,6 +24,12 @@ class ODict:
     def __eq__(self, other):
         return self.lst == other.lst
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+    def __iter__(self):
+        return self.lst.__iter__()
+
     def __getitem__(self, k):
         """
             Returns a list of values matching key.
@@ -33,6 +41,9 @@ class ODict:
                 ret.append(i[1])
         return ret
 
+    def keys(self):
+        return list(set([self._kconv(i[0]) for i in self.lst]))
+
     def _filter_lst(self, k, lst):
         k = self._kconv(k)
         new = []
@@ -53,7 +64,8 @@ class ODict:
             key, they are cleared.
         """
         if isinstance(valuelist, basestring):
-            raise ValueError("ODict valuelist should be lists.")
+            raise ValueError("Expected list of values instead of string. Example: odict['Host'] = ['www.example.com']")
+
         new = self._filter_lst(k, self.lst)
         for i in valuelist:
             new.append([k, i])
@@ -89,13 +101,6 @@ class ODict:
     def items(self):
         return self.lst[:]
 
-    def _get_state(self):
-        return [tuple(i) for i in self.lst]
-
-    @classmethod
-    def _from_state(klass, state):
-        return klass([list(i) for i in state])
-
     def copy(self):
         """
             Returns a copy of this object.
@@ -156,6 +161,18 @@ class ODict:
         self.lst = nlst
         return count
 
+    # Implement the StateObject protocol from mitmproxy
+    def get_state(self, short=False):
+        return [tuple(i) for i in self.lst]
+
+    def load_state(self, state):
+        self.list = [list(i) for i in state]
+
+    @classmethod
+    def from_state(klass, state):
+        return klass([list(i) for i in state])
+
+
 
 class ODictCaseless(ODict):
     """
